import objectdraw.*;
import java.awt.*;
import java.util.Random;

/*
 * 
 * A simple active object that controls a ball that bounces around the
 * canvas and interacts with a boundary and a pong paddle
 *
 * Jim Teresco, Siena College, CSIS 120, Spring 2011
 * Updated for CSC 252, The College of Saint Rose, Fall 2013
 * Updated again Fall 2014 to start at the paddle, not on top
 *
 * $Id: PongBall.java 2264 2013-11-26 04:31:39Z terescoj $
 */

public class PongBall extends ActiveObject {

    // size and speed of falling balls
    private static final int BALLSIZE = 30;
    private static final double MIN_SPEED = 3;
    private static final double MAX_SPEED = 10;
    private static final int DELAY_TIME = 33;

    // the ball controlled by this instance
    private FilledOval ball;

    // the paddle with which we will interact
    private FilledRect paddle;

    // the bricks we are trying to hit
    private BrickCollection bricks;
    
    // how far to fall before stopping and disappearing?
    private double yMax;
    // how far up to go before bouncing off the ceiling?
    private double yMin;
    // and what about the walls?
    private double xMin, xMax;

    // Draw a ball and start it falling.
    public PongBall(FilledRect paddle, FramedRect boundary,
                    BrickCollection bricks,
                    DrawingCanvas aCanvas) {

        // compute/remember our boundaries (of the upper left corner of a ball)
        yMax = boundary.getY()+boundary.getHeight();
        yMin = boundary.getY();
        xMin = boundary.getX();
        xMax = boundary.getX() + boundary.getWidth() - BALLSIZE;

        // draw the ball at its initial position, which is atop the paddle
        ball = new FilledOval(paddle.getX(), paddle.getY() - BALLSIZE - 1,
            BALLSIZE, BALLSIZE, aCanvas);

        // remember the paddle
        this.paddle = paddle;
        
        // remember the bricks
        this.bricks = bricks;

        // activate!
        start();
    }

    // move the ball repeatedly until it falls off screen, bouncing
    // off the paddle and walls along the way
    public void run() {
        double xSpeed, ySpeed;

        // start by moving downward and some amount to the right
        Random r = new Random();

        xSpeed = r.nextDouble() * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
        double initYSpeed = r.nextDouble() * (MAX_SPEED - MIN_SPEED) + MIN_SPEED;
        ySpeed = -initYSpeed;

        // keep moving as long as we haven't fallen off the bottom of
        // the screen
        while (ball.getY() <= yMax) {
            // if we are above the top line, start moving down
            if (ball.getY() < yMin) {
                ySpeed = initYSpeed;
            }

            // if we are in contact with the paddle, start moving up
            if (ball.overlaps(paddle)) {
                ySpeed = -initYSpeed;
            }

            // bounce off side walls
            if (ball.getX() < xMin || ball.getX() > xMax) {
                xSpeed = -xSpeed;
            }

            // bounce off any bricks
            if (bricks.hit(ball)) {
                ySpeed = -ySpeed;
            }
            
            // move a little in the appropriate direction
            ball.move(xSpeed, ySpeed);
            pause(DELAY_TIME);
        }

        ball.removeFromCanvas();
    }
}
