import objectdraw.*;
import java.awt.*;

/*
 * Example PongBricks
 *
 * Jim Teresco, Siena College, CSIS 120, Spring 2012
 * Updated for The College of Saint Rose, CSC 252, Fall 2013
 * Updated again Fall 2014
 *
 * $Id: PongBricks.java 2264 2013-11-26 04:31:39Z terescoj $
 */

public class PongBricks extends WindowController {

    // position and dimensions of the court
    private static final int COURT_LEFT = 50;
    private static final int COURT_TOP = 50;
    private static final int COURT_HEIGHT = 300;
    private static final int COURT_WIDTH = 300;
    private static final int COURT_RIGHT = COURT_LEFT + COURT_WIDTH;

    // dimensions of the paddle
    private static final int PADDLE_WIDTH = 50;
    private static final int PADDLE_HEIGHT = 20;

    private static final int PADDLE_Y = COURT_TOP + COURT_HEIGHT - PADDLE_HEIGHT - 1;

    // how many bricks in the playing area
    private static final int NUM_BRICKS = 10;
    
    private FilledRect paddle;
    private FramedRect boundary; // the boundary of the playing area.

    private BrickCollection bricks;
    
    public void begin() {
        // make the playing area
        boundary = new FramedRect(COURT_LEFT, COURT_TOP,
            COURT_WIDTH, COURT_HEIGHT,
            canvas);

        // make the paddle
        paddle = new FilledRect(COURT_LEFT + (COURT_WIDTH-PADDLE_WIDTH)/2,
            COURT_TOP + COURT_HEIGHT - PADDLE_HEIGHT -1,
            PADDLE_WIDTH, PADDLE_HEIGHT,
            canvas);
            
        // the collection of bricks
        bricks = new BrickCollection(COURT_LEFT, COURT_LEFT+COURT_WIDTH,
                                     COURT_TOP, COURT_TOP+COURT_HEIGHT/2,
                                     NUM_BRICKS, canvas);
    }

    // create a new pong ball if the press is in the playing area
    public void onMousePress(Location point) {

        if (boundary.contains(point)) {
            new PongBall(paddle, boundary, bricks, canvas);
        }
    }

    // move the paddle to follow the mouse's x coordinate
    public void onMouseMove(Location point) {

        if (point.getX() < COURT_LEFT) {
            // place paddle at left edge of the court
            paddle.moveTo(COURT_LEFT, PADDLE_Y);
        } else if (point.getX() + PADDLE_WIDTH > COURT_RIGHT) {
            // place paddle at right edge of the court
            paddle.moveTo(COURT_RIGHT - PADDLE_WIDTH, PADDLE_Y);
        } else {
            // keep the edge of the paddle lined up with the mouse
            paddle.moveTo(point.getX(), PADDLE_Y);
        }

    }
}
